######################################
# building variables
######################################
# debug build?
DEBUG = 1
# optimization
OPT = -O2

CLEAN_BUILD_DIRS = 0


#######################################
# pathes
#######################################
# Build path
DIR_PREFIX 			= ..
DEBUG_DIR = $(DIR_PREFIX)/Debug
DEP_DIR = $(DEBUG_DIR)/.dep
## Drivers sources: recursive scan through all dirs ##
DRIVERS_SOURCES_DIR := $(DIR_PREFIX)/Drivers/
MIDDLEWARES_DIR     := $(DIR_PREFIX)/Middlewares
STARTUP_DIR         := $(DIR_PREFIX)/startup
CORE_DIR            := $(DIR_PREFIX)/Core


# recursive wildcard function:
rwildcard=$(wildcard $1$2) $(foreach d,$(wildcard $1*),$(call rwildcard,$d/,$2))

######################################
# source
######################################
root     := $(DIR_PREFIX)/Core/Src
platform := $(root)/platform_stm32F429
rcore := $(root)/core
cfg   := $(root)/config
apps  := $(root)/apps
boot := $(root)/boot
srv  := $(root)/srv

##################################################
# AUTOMATIC Directories                          #
# Below would be scanned on the directory bases. #
# So all recursive sources and headers in these  #
# folders would be included into the project.    #


## JUNIPER application: recursive scan throw all dirs ##
JUNIPER_SOURCES_DIR := \
$(rcore)/ \
$(platform)/ \
$(srv)/ \
$(cfg)/config/ \
$(apps)/node/ \
$(apps)/tag/ \
$(apps)/listener/ \
$(apps)/tcfm/ \
$(apps)/tcwm/ \
$(apps)/usb2spi/ \
$(apps)/trilat/

## LWIP sources: recursive scan throw all dirs ##
LWIP_SOURCES_DIR :=\
$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/api/ \
$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/core/ \
$(MIDDLEWARES_DIR)/Third_Party/LwIP/system/arch/

## FREERTOS sources: recursive scan throw all dirs ##
FREERTOS_SOURCES_DIR :=\
$(MIDDLEWARES_DIR)/Third_Party/FreeRTOS/Source/

# C/H Scanning results for AUTOMATIC Directories #
JUNIPER_SOURCE_FILES := $(call rwildcard,$(JUNIPER_SOURCES_DIR),*.c)
JUNIPER_HEADER_FILES := $(call rwildcard,$(JUNIPER_SOURCES_DIR),*.h)
JUNIPER_INC_HEADERS  := $(patsubst %,-I%,$(subst //,/,$(sort $(dir $(JUNIPER_HEADER_FILES))))) -I$(cfg)/default_config

DRIVERS_SOURCE_FILES := $(call rwildcard,$(DRIVERS_SOURCES_DIR),*.c)
DRIVERS_HEADER_FILES := $(call rwildcard,$(DRIVERS_SOURCES_DIR),*.h)
DRIVERS_INC_HEADERS  := $(patsubst %,-I%,$(subst //,/,$(sort $(dir $(DRIVERS_HEADER_FILES)))))

LWIP_SOURCE_FILES := $(call rwildcard,$(LWIP_SOURCES_DIR),*.c)
LWIP_HEADER_FILES := $(call rwildcard,$(LWIP_SOURCES_DIR),*.h)
LWIP_INC_HEADERS  := $(patsubst %,-I%,$(subst //,/,$(sort $(dir $(LWIP_HEADER_FILES))))) -I$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/include -I$(MIDDLEWARES_DIR)/Third_Party/LwIP/system

FREERTOS_SOURCE_FILES := $(call rwildcard,$(FREERTOS_SOURCES_DIR),*.c)
FREERTOS_HEADER_FILES := $(call rwildcard,$(FREERTOS_SOURCES_DIR),*.h)
FREERTOS_INC_HEADERS  := $(patsubst %,-I%,$(subst //,/,$(sort $(dir $(FREERTOS_HEADER_FILES)))))

# End of AUTOMATIC Directories                   #
##################################################


##################################################
# Specifying the source files for compilation    #
#
# This will be for support blocks:
# ASM, BSP, LWIP, FREERTOS and USB, and for
# user_application blocks:
#

ASM_SOURCES = $(STARTUP_DIR)/startup_stm32f429xx.s

BSP_SOURCES = \
$(root)/syscalls.c\
$(root)/stm32f4xx_hal_msp.c\
$(root)/stm32f4xx_hal_timebase_tim.c\
$(root)/stm32f4xx_it.c\
$(DRIVERS_SOURCE_FILES)

FREERTOS_SOURCES= $(FREERTOS_SOURCE_FILES)

LWIP_SOURCES= $(LWIP_SOURCE_FILES)\
$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/netif/ethernet.c\
$(MIDDLEWARES_DIR)/Third_Party/LwIP/system/OS/sys_arch.c\
$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/apps/mdns/mdns.c\
$(MIDDLEWARES_DIR)/Third_Party/LwIP/src/apps/tftp/tftp_server.c\
$(root)/app_ethernet.c\
$(root)/ethernetif.c\
$(root)/lwip.c

USB_SOURCES =\
$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Core/Src/usbd_core.c\
$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Core/Src/usbd_ctlreq.c\
$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Core/Src/usbd_ioreq.c\
$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Class/CDC/Src/usbd_cdc.c\
$(root)/usbd_cdc_if.c\
$(root)/usb_device.c\
$(root)/usbd_conf.c\
$(root)/usbd_desc.c


##################################################
# Specifying the Include directories             #

## Juniper User application include files ##
C_INCLUDES :=\
-I$(CORE_DIR)/Inc\
-I$(root)/Inc\
-I$(boot)\
$(JUNIPER_INC_HEADERS)\
$(DRIVERS_INC_HEADERS)\
$(FREERTOS_INC_HEADERS)\
$(LWIP_INC_HEADERS)\
-I$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Class/CDC/Inc\
-I$(MIDDLEWARES_DIR)/ST/STM32_USB_Device_Library/Core/Inc


#######################################
# binaries
#######################################
CC = arm-none-eabi-gcc
AS = arm-none-eabi-gcc -x assembler-with-cpp
CP = arm-none-eabi-objcopy
AR = arm-none-eabi-ar
SZ = arm-none-eabi-size
HEX = $(CP) -O ihex
BIN = $(CP) -O binary -S

#######################################
# APPLICATIONS
#######################################
TDOA_APP_DEFS := -DTDOA_ANCHOR=1 -DUWB_BH_ENABLE
PDOA_APP_DEFS := -DPDOA_NODE=1 -DPDOA_TAG=1 -DCFG_LE_TRILAT=1 -DCFG_LE_TRILAT_UTILS=1


#######################################
# CFLAGS
#######################################
# macros for gcc
AS_DEFS_COMMON 	=
C_DEFS_COMMON  := -D__weak="__attribute__((weak))" -D__packed="__attribute__((__packed__))"
C_DEFS_COMMON  += -DUSE_HAL_DRIVER
C_DEFS_COMMON  += -DSTM32F429xx -DUSE_STM32F4XX_NUCLEO_144


# compile gcc flags
MCU 			= -mcpu=cortex-m4 -mthumb -mfloat-abi=softfp -mfpu=fpv4-sp-d16
ASFLAGS_COMMON	= $(MCU) $(AS_INCLUDES) $(OPT) -Wall -fdata-sections -ffunction-sections
CFLAGS_COMMON 	= $(MCU) $(C_INCLUDES)  $(OPT) -Wall -fdata-sections -ffunction-sections -std=c11

# Generate dependency information
CFLAGS_COMMON  += -MD -MP -MF $(DEP_DIR)/$(@F).d

ifeq ($(DEBUG), 1)
CFLAGS_COMMON  += -g3 -gdwarf-2
endif

# libraries
LIBS = -lc -lm -Wl,--whole-archive -ldwt_uwb_driver-m4-sfp-6.0.7 -Wl,--no-whole-archive #-lrdimon
#-lnosys
#-specs=rdimon.specs
LIBDIR = -L$(DRIVERS_SOURCES_DIR)dwt_uwb_driver/lib
LD_COMMON	     := -mthumb -mcpu=cortex-m4 -mfpu=fpv4-sp-d16 -mfloat-abi=softfp -specs=nosys.specs -specs=nano.specs $(LIBDIR) $(LIBS)


############ using MakefileFWx.mak info from here ############################

TARGET      = $(LOWER_NAME)
BUILD_DIR   = $(DEBUG_DIR)/Debug_$(UPPER_NAME)

# list of objects
OBJECTS = $(addprefix $(BUILD_DIR)/,$(notdir $(C_SOURCES:.c=.o)))
vpath %.c $(sort $(dir $(C_SOURCES)))
# list of ASM program objects
OBJECTS += $(addprefix $(BUILD_DIR)/,$(notdir $(ASM_SOURCES:.s=.o)))
vpath %.s $(sort $(dir $(ASM_SOURCES)))

AS_DEFS := $(AS_DEFS_COMMON)

C_DEFS = $(C_DEFS_COMMON)
C_DEFS += $(C_DEFS_FW)

AS_INCLUDES =

ASFLAGS = $(ASFLAGS_COMMON) $(AS_DEFS)

CFLAGS = $(CFLAGS_COMMON) $(C_DEFS)
ifeq ($(TDOA_APP_DEFS_EN), 1)
CFLAGS += $(TDOA_APP_DEFS)
endif
ifeq ($(PDOA_APP_DEFS_EN), 1)
CFLAGS += $(PDOA_APP_DEFS)
endif

ifeq ($(DEBUG), 1)
CFLAGS += $(CFLAGS_DEBUG)
endif

LDFLAGS = $(LD_COMMON) -T$(LDSCRIPT)  -Wl,-Map=$(BUILD_DIR)/$(TARGET).map,--cref -Wl,--gc-sections

# default action: build all
target: $(BUILD_DIR)/$(TARGET).elf $(BUILD_DIR)/$(TARGET).hex $(BUILD_DIR)/$(TARGET).bin

all: target


$(BUILD_DIR)/%.o: %.c | $(BUILD_DIR)
	@$(CC) -c -o $@ $(CFLAGS) -Wa,-a,-ad,-alms=$(BUILD_DIR)/$(notdir $(<:.c=.lst)) $<

$(BUILD_DIR)/%.o: %.s | $(BUILD_DIR)
	@$(AS) -c -o $@ $(ASFLAGS) $<

$(BUILD_DIR)/$(TARGET).elf: $(OBJECTS)  | $(BUILD_DIR)
	@$(CC) $(OBJECTS) $(LDFLAGS) -o $@
	$(SZ) $@

$(BUILD_DIR)/$(TARGET).hex: $(BUILD_DIR)/$(TARGET).elf | $(BUILD_DIR)
	$(HEX) $< $@

$(BUILD_DIR)/$(TARGET).bin: $(BUILD_DIR)/$(TARGET).elf $(BUILD_DIR)
	$(BIN) $< $@


$(DEBUG_DIR):
	-mkdir $@

$(DEP_DIR): $(DEBUG_DIR)
	-mkdir $@

$(BUILD_DIR): $(DEP_DIR)
	-mkdir $@

post-build:
	-@echo 'Cleaning Build Directory: to disabe, specify this in the Makefile'
ifeq ($(CLEAN_BUILD_DIRS), 1)
	-rm -rf $(BUILD_DIR)/*.o $(BUILD_DIR)/*.lst
endif
	$(SZ) "$(BUILD_DIR)/$(TARGET).elf"
#######################################
# clean up
#######################################
clean:
	-rm -Rf $(DEBUG_DIR)

#######################################
# dependencies
#######################################
-include $(shell mkdir $(DEP_DIR) 2>/dev/null) $(wildcard $(DEP_DIR)/*)

.PHONY: clean all
#.SILENT:



# *** EOF ***
